package ca.ubc.cs.mylyn.monitor.analysis;

import java.io.File;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.mylyn.monitor.core.InteractionEvent;

import ca.ubc.cs.mylyn.monitors.parts.AbstractTextEditorMonitor;
import ca.ubc.cs.mylyn.monitors.parts.analysis.ViewportPayload;

/**
 * List the elements actually viewed; no attempt is made to
 * remove duplicates (or n-licates, really).
 * @author Brian de Alwis
 *
 */
public class ViewedElements {
	protected List<InteractionEvent> events;
	protected List<String> visible = new ArrayList<String>(2000);
	protected int maxAcceptableVisibleLines = 100;

	public static void main(String args[]) {
		if(args.length < 1) {
			System.err.println("use: <program> monitor-files ...");
			return;
		}
		List<InteractionEvent> events = new ArrayList<InteractionEvent>();
		for(String fileName : args) {
			events.addAll(AbstractViewportTrackingAnalyzer.loadEventsFromFile(new File(fileName)));
		}
		Collections.sort(events, AbstractViewportTrackingAnalyzer.getEventComparator());

		ViewedElements analyzer = new ViewedElements(events);
		analyzer.process();
		analyzer.dump(System.out);
	}
	
	public ViewedElements(List<InteractionEvent> events) {
		this.events = events;
	}
	
	public void process() {
		for(int i = 0; i < events.size(); i++) {
			process(events.get(i));
		}
	}

	protected ViewportPayload loadViewportPayload(InteractionEvent event) {
		if ((event.getDelta().startsWith("<")) 
				&& ((JavaUI.ID_CF_EDITOR.equals(event.getOriginId())) 
						|| (JavaUI.ID_CU_EDITOR.equals(event.getOriginId())))
				&& !AbstractTextEditorMonitor.OPENED.equals(event.getNavigation())) {
			// OPENED is sent the first time a file is opened (i.e., a new editor)
			// But it reports the lines that *would* be visible if the entirety
			// could be displayed. Always followed by a BROUGHT_TO_TOP
			// with what is *actually* displayed.
			try {
				return ViewportPayload.load(event.getDelta());
			} catch (RuntimeException e) {
				System.err.println("Exception while parsing viewport payload: "
						+ e.getCause().getMessage());
				System.err.println("   event at: " + event.getDate());
			}
		}
		return null;
	}

	private void process(InteractionEvent event) {
		ViewportPayload vp = loadViewportPayload(event);
		if(vp == null) { return; }
		if(vp.getNumberVisibleLines() > maxAcceptableVisibleLines) {
			System.err.println(event.getDate() + ": reports " + vp.getNumberVisibleLines() + " lines visible; skipping");
		} else {
			for(String handle[] : vp.getHandles().keySet()) {
				visible.add(handle[0] + ":" + handle[1]);
			}
		}
	}

	public void dump(PrintStream out) {
		for(String element : visible) {
			out.println(element);
		}
	}
}
