package ca.ubc.cs.mylyn.monitor.usage.editor;

import java.text.DateFormat;
import java.util.Date;

import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.mylyn.monitor.core.InteractionEvent;
import org.eclipse.swt.graphics.Image;

public class InteractionEventLabelProvider implements ITableLabelProvider, ILabelProvider, IModelListener<InteractionEvent> {
	protected InteractionEvent firstEvent = null;
	protected DateFormat dateFormat = DateFormat.getDateInstance();
	
	public InteractionEventLabelProvider() {
	}

	public void dispose() {
	}

	public Image getColumnImage(Object element, int columnIndex) {
		return null;
	}

	public String getColumnText(Object element, int columnIndex) {
		InteractionEvent event = (InteractionEvent)element;
		switch(columnIndex) {
	    case IInteractionEventDisplayConstants.COLUMN_START_TIME_DELTA:
	    	return firstEvent == null ?  formatDate(event.getDate()) :
	    		formatDateDifference(firstEvent.getDate(), event.getDate());
	    case IInteractionEventDisplayConstants.COLUMN_START_TIME:
	    	return formatDate(event.getDate());
	    case IInteractionEventDisplayConstants.COLUMN_DURATION:
	    	return formatDateDifference(event.getDate(), event.getEndDate());
	    case IInteractionEventDisplayConstants.COLUMN_KIND:
	    	return event.getKind().toString();
	    case IInteractionEventDisplayConstants.COLUMN_CHANGE_DELTA:
	    	return event.getDelta();
	    case IInteractionEventDisplayConstants.COLUMN_ORIGINID:
	    	return event.getOriginId();
	    case IInteractionEventDisplayConstants.COLUMN_STRUCTURE_KIND:
	    	return event.getStructureKind();
	    case IInteractionEventDisplayConstants.COLUMN_NAVIGATION:
	    	return event.getNavigation();
	    case IInteractionEventDisplayConstants.COLUMN_STRUCTURE_HANDLE:
	    	return event.getStructureHandle();
	    case IInteractionEventDisplayConstants.COLUMN_INTEREST_CONTRIBUTION:
	    	return Float.toString(event.getInterestContribution());
		}
		return null;
	}

	private String formatDateDifference(Date beforeDate, Date afterDate) {
		return MonitorHistoryEditor.formatDurationHMS(beforeDate.getTime(), afterDate.getTime()); 
	}

	// Format date 
	private String formatDate(Date date) {
		return dateFormat.format(date);
	}

	public Image getImage(Object element) {
		return null;
	}

	public String getText(Object element) {
		return element.toString();
	}

	public boolean isLabelProperty(Object element, String property) {
		return false;
	}

	public void addListener(ILabelProviderListener listener) {
	}

	public void removeListener(ILabelProviderListener listener) {
	}

	public void modelCleared(ListModel<InteractionEvent> model) {
		firstEvent = null;
	}

	public void elementAdded(ListModel<InteractionEvent> model, InteractionEvent element) {
		firstEvent = model.get(0);
	}

	public void elementRemoved(ListModel<InteractionEvent> model, InteractionEvent element) {
		firstEvent = model.size() > 0 ? model.get(0) : null;
	}

	public void modelElementsChanged(ListModel<InteractionEvent> model) {
		firstEvent = model.size() > 0 ? model.get(0) : null;
	}

	public void setFirstEvent(InteractionEvent event) {
		firstEvent = event;
	}

}
