package ca.ubc.cs.mylyn.monitor.usage.editor;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.ListenerList;

public class ListModel<T> {
	protected List<T> elements;
	protected ListenerList listeners = new ListenerList();

	public ListModel() {
		clear();
	}

	public void clear() {
		elements = new ArrayList<T>(); 
		notifyModelCleared();
	}
	
	public T add(T element) {
		elements.add(element);
		notifyElementAdded(element);
		return element;
	}
	
	public boolean remove(T element) {
		boolean value = elements.remove(element);
		notifyElementRemoved(element);
		return value;
	}

	public void setElements(List<T> contents) {
		elements = new ArrayList<T>(contents);
		notifyElementsChanged();
	}


	public boolean contains(T element) {
		return elements.contains(element);
	}
	
	public List<T> getElements() {
		return elements;
	}
	
	public void addListener(IModelListener<T> l) {
		listeners.add(l);
	}

	public void removeListener(IModelListener<T> l) {
		listeners.remove(l);
	}


	@SuppressWarnings("unchecked")
	protected void notifyModelCleared() {
		for(Object l : listeners.getListeners()) {
			((IModelListener<T>)l).modelCleared(this);
		}
	}

	@SuppressWarnings("unchecked")
	protected void notifyElementsChanged() {
		for(Object l : listeners.getListeners()) {
			((IModelListener<T>)l).modelElementsChanged(this);
		}
	}

	@SuppressWarnings("unchecked")
	protected void notifyElementAdded(T element) {
		for(Object l : listeners.getListeners()) {
			((IModelListener<T>)l).elementAdded(this, element);
		}
	}

	@SuppressWarnings("unchecked")
	protected void notifyElementRemoved(T element) {
		for(Object l : listeners.getListeners()) {
			((IModelListener<T>)l).elementRemoved(this, element);
		}
	}

	public int size() {
		return elements.size();
	}

	public T get(int index) {
		return elements.get(index);
	}

	public T last() {
		if(elements.size() == 0) { return null; }
		return elements.get(elements.size() - 1);
	}

}
