package ca.ubc.cs.mylyn.monitors.parts;

import java.util.Collection;
import java.util.List;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.source.ILineRange;
import org.eclipse.mylyn.internal.monitor.core.util.XmlStringConverter;
import org.eclipse.ui.IWorkbenchPart;

public abstract class AbstractModelBasedTextEditorMonitor<H> extends
		AbstractTextEditorMonitor {
	public static final String TAG_NEWLY_VISIBLE_ELEMENTS = "newlyVisibleElements";
	public static final String TAG_HANDLE = "handle";
	public static final String ATTR_LINES = "lines";

	public AbstractModelBasedTextEditorMonitor() {
	}

	@Override
	protected String describeHandles(String changeType, IWorkbenchPart part,
			List<ILineRange> added, List<ILineRange> removed,
			List<ILineRange> current)
	{
		IDocument document = getSourceViewer(part).getDocument();

		// Report only those elements newly visible because of the change.
		Collection<H> modelElements = 
			determineCorrespondingElements(part, document, added);
		if(modelElements.isEmpty()) { 
			return "<" + TAG_NEWLY_VISIBLE_ELEMENTS + "/>";
		}

		StringBuffer buffer = new StringBuffer();
		buffer.append("<" + TAG_NEWLY_VISIBLE_ELEMENTS + ">\n");
		for(H me : modelElements) {
			String structureKind = getKind(me);
			buffer.append("<");
			buffer.append(TAG_HANDLE);
			buffer.append(" ");
			buffer.append(ATTR_KIND);
			buffer.append("=\"");
			buffer.append(structureKind);
			buffer.append('"');
			ILineRange lr = getLineRange(document, me);
			if(lr != null && lr.getNumberOfLines() > 0) {
				buffer.append(" " + ATTR_LINES + "=\"");
				printInterval(buffer, lr);
				buffer.append('"');
			}
			buffer.append(">");
			buffer.append(XmlStringConverter.convertToXmlString(
					obfuscateHandle(structureKind, getHandle(me))));
			buffer.append("</");
			buffer.append(TAG_HANDLE);
			buffer.append(">\n");
		}
		buffer.append("</");
		buffer.append(TAG_NEWLY_VISIBLE_ELEMENTS);
		buffer.append(">\n");
		return buffer.toString();
	}
	
	protected abstract Collection<H> determineCorrespondingElements(IWorkbenchPart part,
			IDocument document, List<ILineRange> change);
	protected abstract ILineRange getLineRange(IDocument doc, H me);
	protected abstract String getKind(H me);
	protected abstract String getHandle(H me);
}
