package ca.ubc.cs.mylyn.monitors.parts.platform;

import org.eclipse.core.runtime.Platform;
import org.eclipse.mylyn.internal.monitor.usage.UiUsageMonitorPlugin;
import org.eclipse.ui.IStartup;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

import ca.ubc.cs.mylyn.monitors.parts.jdt.JavaEditorViewportMonitor;
import ca.ubc.cs.mylyn.monitors.parts.jdt.PackageExplorerMonitor;

/**
 * The activator class controls the plug-in life cycle
 */
public class Activator extends AbstractUIPlugin implements IStartup {

	// The plug-in ID
	public static final String PLUGIN_ID = "ca.ubc.cs.mylyn.monitors.parts.examples";

	// The shared instance
	private static Activator plugin;
	
	/**
	 * The constructor
	 */
	public Activator() {
	}

	JavaEditorViewportMonitor javaEditorMonitor;
	DefaultTextEditorViewportMonitor textEditorMonitor;
	PackageExplorerMonitor packageExplorerMonitor;
	ProjectExplorerMonitor projectExplorerMonitor;
	ConsoleScrollingMonitor outlineMonitor;
	
	public void earlyStartup() {
		if(installMonitors()) {
			System.out.println(">>> Installing example part monitors");
			javaEditorMonitor = new JavaEditorViewportMonitor();
			textEditorMonitor = new DefaultTextEditorViewportMonitor();
			packageExplorerMonitor = new PackageExplorerMonitor();
			projectExplorerMonitor = new ProjectExplorerMonitor();
			outlineMonitor = new ConsoleScrollingMonitor();
			UiUsageMonitorPlugin.getDefault().addMonitoringLifecycleListener(javaEditorMonitor);
			UiUsageMonitorPlugin.getDefault().addMonitoringLifecycleListener(textEditorMonitor);
			UiUsageMonitorPlugin.getDefault().addMonitoringLifecycleListener(packageExplorerMonitor);
			UiUsageMonitorPlugin.getDefault().addMonitoringLifecycleListener(projectExplorerMonitor);
			UiUsageMonitorPlugin.getDefault().addMonitoringLifecycleListener(outlineMonitor);
		}
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		if(installMonitors()) {
			System.out.println(">>> Removing example part monitors");
		}
		if(javaEditorMonitor != null) {
			UiUsageMonitorPlugin.getDefault().removeMonitoringLifecycleListener(javaEditorMonitor);
			javaEditorMonitor = null;
		}
		if(textEditorMonitor != null) {
			UiUsageMonitorPlugin.getDefault().removeMonitoringLifecycleListener(textEditorMonitor);
			textEditorMonitor = null;
		}
		if(packageExplorerMonitor != null) {
			UiUsageMonitorPlugin.getDefault().removeMonitoringLifecycleListener(packageExplorerMonitor);
			packageExplorerMonitor = null;
		}
		if(projectExplorerMonitor != null) {
			UiUsageMonitorPlugin.getDefault().removeMonitoringLifecycleListener(projectExplorerMonitor);
			projectExplorerMonitor = null;
		}
		if(outlineMonitor != null) {
			UiUsageMonitorPlugin.getDefault().removeMonitoringLifecycleListener(outlineMonitor);
			outlineMonitor = null;
		}

		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static Activator getDefault() {
		return plugin;
	}

	protected boolean installMonitors() {
		String value = Platform.getDebugOption(PLUGIN_ID + "/startPartMonitors");
        if(value == null) { return false; }
        return "true".equalsIgnoreCase(value) || "yes".equalsIgnoreCase(value)
        	|| value.equals("1");
	}
}
