package ca.ubc.cs.mylyn.monitors.parts.platform;

import org.eclipse.mylyn.monitor.core.InteractionEvent;
import org.eclipse.mylyn.monitor.ui.IMylarMonitorLifecycleListener;
import org.eclipse.mylyn.monitor.ui.MonitorUiPlugin;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.widgets.ScrollBar;
import org.eclipse.swt.widgets.Scrollable;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbenchPart;

import ca.ubc.cs.mylyn.monitors.parts.IPartScrollingListener;
import ca.ubc.cs.mylyn.monitors.parts.PartDispatcher;

/**
 * Example monitor using {@link PartDispatcher}'s part-scrolling
 * functionality.  This is useful for parts that don't provide ready access
 * to their UI bits, or whose UI changes dynamically.  This example
 * listens for vertical scrolling changes in the console view. 
 * @author Brian de Alwis
 */
public class ConsoleScrollingMonitor implements IPartScrollingListener,
		IMylarMonitorLifecycleListener {
	// IConsoleConstants.ID_CONSOLE_VIEW: we have enough dependencies anyways
	protected final static String viewId = "org.eclipse.ui.console.ConsoleView";
	
	public ConsoleScrollingMonitor() {
	}

	protected void notify(InteractionEvent event) {
		MonitorUiPlugin.getDefault().notifyInteractionObserved(event);
	}

	public void partScrolled(String partId, IWorkbenchPart part,
			Scrollable scrollable, ScrollBar scrollBar) {
		if((scrollBar.getStyle() & SWT.V_SCROLL) != 0) {
			String changeDelta = createChangeDelta("scrolling",
					part, scrollable, scrollBar);
			notify(new InteractionEvent(InteractionEvent.Kind.SELECTION,
					"", "", part.getSite().getId(), "scrolling", changeDelta, 0f));
		}
	}
	
	protected String createChangeDelta(String changeType, IWorkbenchPart part,
			Scrollable scrollable, ScrollBar scrollBar) {
		// Assumes we're a vertical scroll
		int scrollSize = scrollBar.getSize().y;	// since V_SCROLL
		StringBuffer buf = new StringBuffer("<");
		buf.append(changeType);
		if(scrollable instanceof Text) {
			Text text = (Text)scrollable;
			// line indices are 0-relative, but the end should be top + offset - 1
			buf.append(" visible=\"");
			buf.append(text.getTopIndex() + 1);
			buf.append(":");
			buf.append(text.getTopIndex() + (text.getClientArea().height) /
							text.getLineHeight());
			buf.append("\"");
		} else if(scrollable instanceof StyledText) {
			StyledText text = (StyledText)scrollable;
			// line indices are 0-relative, but the end should be top + offset - 1
			buf.append(" visible=\"");
			buf.append(text.getTopIndex() + 1);
			buf.append(":");
			buf.append(text.getTopIndex() + (text.getClientArea().height) /
							text.getLineHeight());
			buf.append("\"");
		} else {
			buf.append(" visiblePixels=\""); 
			buf.append(scrollBar.getSelection()); 
			buf.append(":");
			buf.append(scrollBar.getSelection() + scrollSize);
			buf.append("\"");
		}
		buf.append("/>");
		return buf.toString();
	}

	public void startMonitoring() {
		System.out.println(getClass().getName() + ": startMonitoring()");
		PartDispatcher.getDefault().addScrollingListener(viewId, this);
	}

	public void stopMonitoring() {
		System.out.println(getClass().getName() + ": stopMonitoring()");
		PartDispatcher.getDefault().removeScrollingListener(viewId, this);
	}

}
